import React, { useCallback, useState } from 'react';
import {
    DropzoneOptions,
    ErrorCode,
    FileError,
    FileRejection,
} from 'react-dropzone';
import { AlertTitle } from '@mui/material';

export const useDropzoneFiles = () => {
    const [files, setFiles] = useState<File[]>([]);

    const onDrop = useCallback<NonNullable<DropzoneOptions['onDrop']>>(
        async (acceptedFiles) => {
            setFiles((files) => [...files, ...acceptedFiles]);
        },
        []
    );

    const [errors, setErrors] = useState<JSX.Element[] | null>(null);

    const onDropRejected = useCallback<
        NonNullable<DropzoneOptions['onDropRejected']>
    >(async (rejectedFiles) => {
        setErrors(alertTitlesFrom(rejectedFiles));
    }, []);

    const deleteFiles = () => {
        setFiles([]);
    };
    const deleteFile = (toBeDeleted: File) => {
        setFiles((files) => files.filter((f) => f !== toBeDeleted));
    };
    const clearErrors = () => setErrors(null);

    return {
        files,
        errors,
        onDrop,
        onDropRejected,
        clearErrors,
        deleteFile,
        deleteFiles,
    };
};

const fileErrorToString = (e: FileError) => {
    switch (e.code) {
        case ErrorCode.FileInvalidType:
            return 'File has an invalid type';
        case ErrorCode.FileTooLarge:
            return 'File is larger than 2 MB';
    }
};

const alertTitlesFrom = (rejectedFiles: FileRejection[]): JSX.Element[] => {
    return [
        <AlertTitle>The following files were rejected:</AlertTitle>,
        ...rejectedFiles.map((f) => {
            return (
                <AlertTitle>
                    · {f.file.name} -{' '}
                    {f.errors
                        .map((e) => {
                            return fileErrorToString(e);
                        })
                        .join(', ')}
                </AlertTitle>
            );
        }),
    ];
};
